@@ -1,7 +1,7 @@
 /*
 This file is part of the GhostDriver by Ivan De Marino <http://ivandemarino.me>.
 
-Copyright (c) 2014, Ivan De Marino <http://ivandemarino.me>
+Copyright (c) 2012-2014, Ivan De Marino <http://ivandemarino.me>
 Copyright (c) 2014, Alex Anderson <@alxndrsn>
 All rights reserved.
 
@@ -138,10 +138,9 @@ ghostdriver.WebElementReqHand = function(idOrElement, session) {
     },
 
     _getLocationResult = function(req) {
-        return _protoParent.getSessionCurrWindow.call(this, _session, req).evaluate(
-            require("./webdriver_atoms.js").get("execute_script"),
-            "return (" + require("./webdriver_atoms.js").get("get_location") + ")(arguments[0]);",
-            [_getJSON()]);
+        return JSON.parse(_protoParent.getSessionCurrWindow.call(this, _session, req).evaluate(
+            require("./webdriver_atoms.js").get("get_location"),
+            _getJSON()));
     },
 
     _getLocation = function(req) {
@@ -164,11 +163,23 @@ ghostdriver.WebElementReqHand = function(idOrElement, session) {
         res.respondBasedOnResult(_session, req, locationRes);
     },
 
+    // scrolls the element into view
     _getLocationInViewResult = function (req) {
-        return _protoParent.getSessionCurrWindow.call(this, _session, req).evaluate(
-            require("./webdriver_atoms.js").get("execute_script"),
-            "return (" + require("./webdriver_atoms.js").get("get_location_in_view") + ")(arguments[0]);",
-            [_getJSON()]);
+        var currWindow = _protoParent.getSessionCurrWindow.call(this, _session, req),
+            frameOffset = _session.getFrameOffset(currWindow),
+            locationRes;
+
+        locationRes = JSON.parse(_protoParent.getSessionCurrWindow.call(this, _session, req).evaluate(
+            require("./webdriver_atoms.js").get("get_location_in_view"),
+            _getJSON(), true));
+
+        if(locationRes && locationRes.status !== 0) {
+            return locationRes;
+        }
+
+        locationRes.value.x += frameOffset.left;
+        locationRes.value.y += frameOffset.top;
+        return locationRes;
     },
 
     _getLocationInView = function (req) {
@@ -213,11 +224,61 @@ ghostdriver.WebElementReqHand = function(idOrElement, session) {
     _getSizeCommand = function (req, res) {
         var sizeRes = _getSizeResult(req);
 
-        _log.debug("_getSizeCommand", JSON.stringify(sizeRes))
+        _log.debug("_getSizeCommand", JSON.stringify(sizeRes));
 
         res.respondBasedOnResult(_session, req, sizeRes);
     },
 
+
+    // coordinates for native click events
+    _getPosition = function(req) {
+        var currWindow = _protoParent.getSessionCurrWindow.call(this, _session, req),
+            locationRes = _getLocationInViewResult(req),
+            sizeRes = JSON.parse(_getSizeResult(req));
+
+        if(locationRes && locationRes.status !== 0) {
+            return locationRes;
+        }
+
+        if(sizeRes && sizeRes.status !== 0) {
+            return sizeRes;
+        }
+
+        return {status: 0,
+            x: (locationRes.value.x + sizeRes.value.width/2),
+            y: (locationRes.value.y + sizeRes.value.height/2)};
+    },
+
+    _nativeClick = function(req) {
+        var currWindow = _protoParent.getSessionCurrWindow.call(this, _session, req),
+        clickRes,
+        coords;
+
+        currWindow.evaluate(require("./webdriver_atoms.js").get("scroll_into_view"),_getJSON());
+        var interactableRes = currWindow.evaluate(require("./webdriver_atoms.js").get("is_displayed"), _getJSON());
+        interactableRes = JSON.parse(interactableRes);
+        if (interactableRes && interactableRes.status !== 0) {
+            return interactableRes;
+        }
+
+        if (!interactableRes.value) {
+            return {
+                status: _errors.FAILED_CMD_STATUS_CODES.ElementNotVisible,
+                value: {message: "Element is not displayed"}
+            };
+        }
+
+        coords = _getPosition();
+        if (coords && coords.status !== 0) {
+            return coords;
+        }
+        _session.inputs.mouseMove(_session, coords);
+        _session.inputs.mouseButtonClick(_session, "click", "left");
+        _log.debug("Click at: " + JSON.stringify(coords));
+
+        return {status: 0, value: null};
+    },
+
     _normalizeSpecialChars = function(str) {
         var resultStr = "",
             i, ilen;
@@ -253,7 +314,18 @@ ghostdriver.WebElementReqHand = function(idOrElement, session) {
             currWindow = _protoParent.getSessionCurrWindow.call(this, _session, req),
             typeRes,
             text,
-            fsModule = require("fs");
+            isFileInputRes,
+            isContentEditableRes,
+            fsModule = require("fs"),
+            abortCallback = false,
+            multiFileText;
+
+        isFileInputRes = currWindow.evaluate(require("./webdriver_atoms.js").get("is_file_input"), _getJSON());
+        isFileInputRes = JSON.parse(isFileInputRes);
+        if (isFileInputRes && isFileInputRes.status !== 0) {
+            res.respondBasedOnResult(_session, req, isFileInputRes);
+            return;
+        }
 
         // Ensure all required parameters are available
         if (typeof(postObj) === "object" && typeof(postObj.value) === "object") {
@@ -261,42 +333,82 @@ ghostdriver.WebElementReqHand = function(idOrElement, session) {
             text = postObj.value.join("");
 
             // Detect if it's an Input File type (that requires special behaviour), and the File actually exists
-            if (_getTagName(currWindow).toLowerCase() === "input" &&
-                _getAttribute(currWindow, "type").toLowerCase() === "file" &&
-                fsModule.exists(text)) {
-                // Register a one-shot-callback to fill the file picker once invoked by clicking on the element
-                currWindow.setOneShotCallback("onFilePicker", function(oldFile) {
-                    // Send the response as soon as we are done setting the value in the "input[type=file]" element
-                    setTimeout(function() {
-                        res.respondBasedOnResult(_session, req, typeRes);
-                    }, 1);
+            if (isFileInputRes.value) {
 
-                    return text;
-                });
+                // split files by \n like chromedriver
+                multiFileText = text.split("\n");
+
+                // abort if file does not exist
+                for (var i = 0; i < multiFileText.length; ++i) {
+                    if (!fsModule.exists(multiFileText[i])) {
+                        _log.debug("File does not exist: " + multiFileText[i]);
+                        res.success(_session.getId());
+                        return;
+                    }
+                }
+
+                // this indirectly clicks on the head element
+                // hack to workaround phantomjs uploadFile api which requires a selector
+                currWindow.uploadFile("head", multiFileText);
 
                 // Click on the element!
-                typeRes = currWindow.evaluate(require("./webdriver_atoms.js").get("click"), _getJSON());
+                typeRes = _nativeClick();
+                res.respondBasedOnResult(_session, req, typeRes);
+                return;
+
             } else {
                 // Normalize for special characters
                 text = _normalizeSpecialChars(text);
 
                 // Execute the "type" atom on an empty string only to force focus to the element.
                 // TODO: This is a hack that needs to be corrected with a proper method to set focus.
-                typeRes = currWindow.evaluate(require("./webdriver_atoms.js").get("type"), _getJSON(), "");
-
-                // Send keys to the page, using Native Events
-                _session.inputs.sendKeys(_session, text);
-
-                // Only clear the modifier keys if this was called using element.sendKeys().
-                // Calling this from the Advanced Interactions API doesn't clear the modifier keys.
-                if (req.urlParsed.file === _const.VALUE) {
-                    _session.inputs.clearModifierKeys(_session);
+                isContentEditableRes = currWindow.evaluate(require("./webdriver_atoms.js").get("is_content_editable"), _getJSON());
+                isContentEditableRes = JSON.parse(isContentEditableRes);
+                if (isContentEditableRes && isContentEditableRes.status !== 0) {
+                    res.respondBasedOnResult(_session, req, isContentEditableRes);
+                    return;
+                }
+                if (isContentEditableRes.value) {
+                    // must use native click to focus on content editable element
+                    typeRes = _nativeClick();
+                } else {
+                    typeRes = currWindow.evaluate(require("./webdriver_atoms.js").get("type"), _getJSON(), "");
+                    typeRes = JSON.parse(typeRes);
                 }
 
-                currWindow.waitIfLoading(function() {
-                    // Return the result of this typing
+                if (typeRes && typeRes.status !== 0) {
+                    abortCallback = true;           //< handling the error here
                     res.respondBasedOnResult(_session, req, typeRes);
-                });
+                    return;
+                }
+
+                currWindow.execFuncAndWaitForLoad(function() {
+
+                        // Send keys to the page, using Native Events
+                        _session.inputs.sendKeys(_session, text);
+
+                        // Only clear the modifier keys if this was called using element.sendKeys().
+                        // Calling this from the Advanced Interactions API doesn't clear the modifier keys.
+                        if (req.urlParsed.file === _const.VALUE) {
+                            _session.inputs.clearModifierKeys(_session);
+                        }
+                    },
+                    function(status) {                   //< onLoadFinished
+                        // Report Load Finished, only if callbacks were not "aborted"
+                        if (!abortCallback) {
+                            res.success(_session.getId());
+                        }
+                    },
+                    function(errMsg) {
+                        var errCode = errMsg === "timeout"
+                            ? _errors.FAILED_CMD_STATUS_CODES.Timeout
+                            : _errors.FAILED_CMD_STATUS_CODES.UnknownError;
+
+                        // Report Load Error, only if callbacks were not "aborted"
+                        if (!abortCallback) {
+                            _errors.handleFailedCommandEH(errCode, "Pageload initiated by click failed. Cause: " + errMsg, req, res, _session);
+                        }
+                    });
             }
             return;
         }
@@ -305,16 +417,9 @@ ghostdriver.WebElementReqHand = function(idOrElement, session) {
     },
 
     _getNameCommand = function(req, res) {
-        var result = _protoParent.getSessionCurrWindow.call(this, _session, req).evaluate(
-                require("./webdriver_atoms.js").get("execute_script"),
-                "return arguments[0].tagName;",
-                [_getJSON()]);
-
-        // Convert value to a lowercase string as per WebDriver JSONWireProtocol spec
-        // @see http://code.google.com/p/selenium/wiki/JsonWireProtocol#/session/:sessionId/element/:id/name
-        if(result.status === 0) {
-            result.value = result.value.toLowerCase();
-        }
+        var result = JSON.parse(_protoParent.getSessionCurrWindow.call(this, _session, req).evaluate(
+                require("./webdriver_atoms.js").get("get_name"),
+                _getJSON()));
 
         res.respondBasedOnResult(_session, req, result);
     },
@@ -348,10 +453,9 @@ ghostdriver.WebElementReqHand = function(idOrElement, session) {
         var result;
 
         if (typeof(req.urlParsed.file) === "string" && req.urlParsed.file.length > 0) {
-            result = _protoParent.getSessionCurrWindow.call(this, _session, req).evaluate(
-                require("./webdriver_atoms.js").get("execute_script"),
-                "return arguments[0].isSameNode(arguments[1]);",
-                [_getJSON(), _getJSON(req.urlParsed.file)]);
+            result = JSON.parse(_protoParent.getSessionCurrWindow.call(this, _session, req).evaluate(
+                require("./webdriver_atoms.js").get("equals"),
+                _getJSON(), _getJSON(req.urlParsed.file)));
 
             res.respondBasedOnResult(_session, req, result);
             return;
@@ -372,35 +476,24 @@ ghostdriver.WebElementReqHand = function(idOrElement, session) {
                 // If Submit was NOT positive, status will be set to something else than '0'
                 submitRes = JSON.parse(submitRes);
                 if (submitRes && submitRes.status !== 0) {
-                    _log.warn("_postSubmitCommand", "Error bailout");
                     abortCallback = true;           //< handling the error here
                     res.respondBasedOnResult(_session, req, submitRes);
-                } else {
-                    _log.warn("_postSubmitCommand", "Request submitted, waiting for response/(load");
-                };
+                }
             },
             function(status) {                   //< onLoadFinished
                 // Report about the Load, only if it was not already handled
                 if (!abortCallback) {
-                    _log.warn("_postSubmitCommand", "Status: Success");
                     res.success(_session.getId());
                 }
             },
             function(errMsg) {
                 var errCode = errMsg === "timeout"
-                    ? _errors.FAILED_CMD_STATUS.TIMEOUT
-                    : _errors.FAILED_CMD_STATUS.UNKNOWN_ERROR;
-                _log.warn("_postSubmitCommand", "Error: "+errMsg);
+                    ? _errors.FAILED_CMD_STATUS_CODES.Timeout
+                    : _errors.FAILED_CMD_STATUS_CODES.UnknownError;
 
                 // Report Submit Error, only if callbacks were not "aborted"
                 if (!abortCallback) {
-                    _errors.handleFailedCommandEH(
-                        errCode,
-                        "Submit failed: " + errMsg,
-                        req,
-                        res,
-                        _session,
-                        "WebElementReqHand");
+                    _errors.handleFailedCommandEH(errCode, "Submit failed: " + errMsg, req, res, _session);
                 }
             });
     },
@@ -430,18 +523,12 @@ ghostdriver.WebElementReqHand = function(idOrElement, session) {
             },
             function(errMsg) {
                 var errCode = errMsg === "timeout"
-                    ? _errors.FAILED_CMD_STATUS.TIMEOUT
-                    : _errors.FAILED_CMD_STATUS.UNKNOWN_ERROR;
+                    ? _errors.FAILED_CMD_STATUS_CODES.Timeout
+                    : _errors.FAILED_CMD_STATUS_CODES.UnknownError;
 
                 // Report Load Error, only if callbacks were not "aborted"
                 if (!abortCallback) {
-                    _errors.handleFailedCommandEH(
-                        errCode,
-                        "Click failed: " + errMsg,
-                        req,
-                        res,
-                        _session,
-                        "WebElementReqHand");
+                    _errors.handleFailedCommandEH(errCode, "Pageload initiated by click failed. Cause: " + errMsg, req, res, _session);
                 }
             });
     },
@@ -489,14 +576,6 @@ ghostdriver.WebElementReqHand = function(idOrElement, session) {
         return JSON.parse(result).value;
     },
 
-    _getTagName = function(currWindow) {
-        var result = currWindow.evaluate(
-                require("./webdriver_atoms.js").get("execute_script"),
-                "return arguments[0].tagName;",
-                [_getJSON()]);
-
-        return result.value;
-    },
 
     /**
      * This method can generate any Element JSON: just provide an ID.
